# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: CommonProcs.tcl,v 1.19 2003/10/10 21:45:44 tsmith Exp $

#---------------------------------------------------------------------------#
# Common Procedures -- used for making ListBoxes etc..			    #
#---------------------------------------------------------------------------#
#
#	MakeListBox -- routine to make a double scrolled list box
#	ColorLevelToRGB -- routine to convert McIDAS color level to a RGB value
#	ColorLevelToName -- routine to convert McIDAS color level to a Name
#       ColorSelector -- This displays and allows selection of all of the user defined colors 
#	ColorChooser -- choose a color level and return that level
#       InverseColor -- computes the inverse of the color sent in and returns that
#	GetLocalDataPath -- build a path into local data directory
#---------------------------------------------------------------------------#
# MakeListBox { Parent } -- routine to make a double scrolled list box      #
#	-- Parent -- frame to insert the list box into			    #
#---------------------------------------------------------------------------#
proc MakeListBox { Parent Title1 Title2 } \
  {      
  global ListBoxSelections
  
  set fontset {-adobe-courier-bold-r-normal--*-90*}
  if {[string length $Title1]} \
    {
    label $Parent.labelTitle -text $Title1 -anchor w -font $fontset
    pack $Parent.labelTitle -side top -fill x 
    }
  if {[string length $Title2]} \
    {
    label $Parent.labelTitle2 -text $Title2 -anchor w -font $fontset
    pack $Parent.labelTitle2 -side top -fill x 
    }
    
  listbox $Parent.lList -yscrollcommand [list $Parent.sY set] \
                        -xscrollcommand [list $Parent.sX set] \
                        -width 50 \
  	                -selectforeground black -selectbackground "#d0d0d0" \
		        -background white -selectborderwidth 2
			
  scrollbar $Parent.sY -orient vertical -command [list $Parent.lList yview]

  # -- need an extra frame here to hold the bottom scroll bar and some padding 
    frame $Parent.fBottom
    scrollbar $Parent.sX -orient horizontal -command [list $Parent.lList xview]
    set pad [expr [$Parent.sY cget -width] +2 * \
              ([$Parent.sY cget -highlightthickness] + \
               [$Parent.sY cget -bd])]
    frame $Parent.fPad -width $pad -height $pad
  # ---pack em
    pack $Parent.fBottom -side bottom -fill x
    pack $Parent.fPad -in $Parent.fBottom -side right
    pack $Parent.sX -in $Parent.fBottom -side bottom -fill x
    pack $Parent.sY -side right -fill y         	          
    pack $Parent.lList -side left -fill both -expand true
    
  # --- set up bindings to try and keep the items in the list selected
    set ListBoxSelections($Parent.lList) ""
    bind $Parent.lList <Leave> { 
       # Save the index list
        set ListBoxSelections(%W) [%W curselection]
       }
    bind $Parent.lList <Visibility> { 
       # -- visibility is changing -- so highlight the selected index(s)
       #focus %W
       foreach line $ListBoxSelections(%W) \
         {
	 %W selection set $line
	 }
       }
    bind $Parent.lList <Enter> { 
       # -- Entered the LB again
#       foreach line $ListBoxSelections(%W) \
#         {
#	 %W selection set $line
#	 }
       }
 
  return $Parent.lList  
  }
#-----------------------------------------------------------------------------#
# ColorLevelToRGB { lev } -- converts a McIDAS color level to an RGB value    #
#-----------------------------------------------------------------------------#
proc ColorLevelToRGB { lev } \
  {
  set guout [open "|gu.k TAB" r]
  while {[gets $guout guline] > -1} \
    {
    set level [string trim [string range $guline 3 4]]
    if {$level == $lev} \
      {
      set red [string range $guline 35 38]
      set green [string range $guline 31 33]
      set blue [string range $guline 24 26]
      # eat the rest of the pipe 
      while {[gets $guout guline] > -1} { }
      catch { close $guout }
      return [format "#%02x%02x%02x" $red $green $blue]
      }
    }
  }
#-----------------------------------------------------------------------------#
# ColorLevelToName { lev } -- converts a McIDAS color level to an RGB value   #
#-----------------------------------------------------------------------------#
proc ColorLevelToName { lev } \
  {
  set guout [open "|gu.k TAB" r]
  while {[gets $guout guline] > -1} \
    {
    set level [string trim [string range $guline 3 4]]
    if {$level == $lev} \
      {
      set Name [string trim [string range $guline 8 17]]
      if { ![string length $Name] } {set Name "USER" }
      # eat the rest of the pipe 
      while {[gets $guout guline] > -1} { }
      catch { close $guout }
      return $Name
      }
    }
  }

#---------------------------------------------------------------------
# ColorSelector -- This displays all of the user defined colors 
#               -- a color in the format {#rrggbb} is returned
#---------------------------------------------------------------------
proc ColorSelect { DefaultColor } \
   {
   global colorIndex
   global tk_version

   # Tk widget commands moved to namespaces with version 8.1 and higher
   # Expose the old private Tk commands.
   if { $tk_version >= 8.1 } \
   {
	::tk::unsupported::ExposePrivateCommand tk*
   }
   
   set colorIndex -2  ;# default to the fourht color (RED) in case of early exit
   #-- make a toplevel window -- new dialog box
     toplevel .colors 
     wm title .colors "Color Selection"
   
   #-- now get the entire list -- use the "mcidas" command to get the correct path -- read only
      set collist [open [mcidas pathname MCRGB.TXT] r]

   # The following are used to hold the color data
   # -names   	-- color names
   # -reds      	-- red values
   # -greens   	-- green values
   # -blues	-- blue values
   # -colors	-- counter of the number of colors

    set colors 0
    while {[gets $collist line] > -1}\
        {
        set reds($colors) [lindex $line 0]
        set greens($colors) [lindex $line 1]
        set blues($colors) [lindex $line 2]
        set names($colors) [lindex $line 3]
        incr colors
        }
    catch { close $collist }
    #-- now create a ton of buttons to hold all of the colors -- do it in rows of 3
       set y [expr $colors/3 + {$colors%3}]     	;# how many rows
   
    #-- main frame
    frame .colors.colorFrame -borderwidth 2 -relief raised 

    #-- prep to build the buttons now
       set total 0		;# used to move through the entire list until total=colors
       set rows 0
       while { $rows < $y } \
         {
         # --- Build a frame here to hold three buttons
         frame .colors.colorFrame.frame$rows
         set x 0
         while {$x<3 && $total<$colors} \
           {
           #-- build the buttons color here from red/green/blue values
           set color [format "#%02x%02x%02x" $reds($total) $greens($total) $blues($total)]
           
           button .colors.colorFrame.frame$rows.$total -background $color -foreground [InverseColor $color] \
              -activeforeground [InverseColor $color] -activebackground $color \
              -highlightthickness 0 -text $names($total) -relief raised -width 10\
              -command "set colorIndex $total; destroy .colors "
                
           pack .colors.colorFrame.frame$rows.$total -side left -padx 5 -pady 3  
           incr x
           incr total
           }
         pack .colors.colorFrame.frame$rows -side top   
         incr rows   
         }
    #-- place the frame here
      pack .colors.colorFrame -side top -fill x 
           
    #-- now build the button to select the tk_chooseColor dialog      
      frame .colors.framePicker -relief sunken -borderwidth 2
      button .colors.framePicker.buttonColorPicker \
                -background $DefaultColor \
                -foreground [InverseColor $DefaultColor] \
                -activeforeground [InverseColor $DefaultColor] \
                -activebackground $DefaultColor \
                -highlightthickness 0 \
                -text "Color Picker" \
                -relief raised \
                -command "set colorIndex -1; destroy .colors "

      pack .colors.framePicker.buttonColorPicker -side top -pady 8 -padx 8
      pack .colors.framePicker -side top -padx 20 -pady 20          

    placepopup .colors
       
    # now wait for the window to be destroyed and then return
      tkwait window .colors
    # will hold -1 when the tk_chooseColor dialog needs to be run 
     if { $colorIndex==-2 } \
       {
       # Operation was canceled
       return $DefaultColor   
       } \
     elseif { $colorIndex != -1 } \
       { 
       set color [format "#%02x%02x%02x" $reds($colorIndex) $greens($colorIndex) $blues($colorIndex)] 
       return $color
       } \
     else \
       {
       set color [tk_chooseColor -initialcolor $DefaultColor]
       # if cancel was pressed then the return value will be "" so just return the DefaultColor
       if {[string first "#" $color] == -1} { return $DefaultColor} else {return $color}
       }
    }
#---------------------------------------------------------------------
# InverseColor (color) -- computes the inverse of the color sent in and returns that
#---------------------------------------------------------------------
proc InverseColor { color } \
   {
   scan $color "#%02x%02x%02x" r g b         

   if {$r + $g + $b >  128*3 } \
      {
      set r 0
      set g 0
      set b 0
      } \
   else \
      {
      set r 0xff
      set g 0xff
      set b 0xff
      }
   
   return [format "#%02x%02x%02x" $r $g $b]
   }
#--------------------------------------------------------------------
# StripNonNumeric { inputString } -- strips all non numerals in inputString and
#		returns the resulting number
#--------------------------------------------------------------------
proc StripNonNumeric { inputString } \
  {
  
  set position 0 	  ;# current position in the input string
  set output ""		  ;# This will be where the numeric string is built
  
  while { $position <= [string length $inputString]}\
    {
    if { [string match {[0-9]*} [string range $inputString $position $position]]} \
      { 
      append output [string range $inputString $position $position] 
      }
    incr position  
    }
  return $output
  }


#--------------------------------------------------------------------
# GetLocalDataPath { DataFile } -- builds the correct path to the GUI 
#               directory and appends the file name
#--------------------------------------------------------------------
proc GetLocalDataPath { DataFile } \
{
  global env

  #--- if we're a mcwish script, things are easy
  if [running_mcwish] \
  {
	return [mcidas pathname $DataFile]
  } \
  else \
  {
    	return "Not running MCWISH, cannot search MCPATH nor LWPATH.NAM."
  }
}


#--------------------------------------------------------------------#
# ColorChooser { lev } -- returns a color level selected by the user #
#   -- lev -- current level of choice (default)                      #
#--------------------------------------------------------------------#
proc ColorChooser { lev } \
  {
  global CCLevel
  global ColorFlags
  
  set CCLevel $lev  
  if { [info exists ColorFlags(Levels)] } \
    { set showlevel $ColorFlags(Levels) } \
  else\
    { set showlevel 1 }

  toplevel .tlChooser -relief raised -borderwidth 3
  wm title .tlChooser "Graphics Color Chooser"

  # determine how many graphics levels that there are
  set GraphicsLevels [UCpeek [ucLocation number_graphics]]

  #--- create the color list
    set guout [open "|gu.k TAB " r]
    while {[gets $guout guline]>-1} \
      {
      set level [lindex $guline 0]
      if { $level>=0 && $level<=$GraphicsLevels } \
        {
        set red [string range $guline 35 38]
        set green [string range $guline 31 33]
        set blue [string range $guline 24 26]
        set name [string trim [string range $guline 8 17]]
        set ColorList($level) [format "#%02x%02x%02x $name" $red $green $blue]
        }
      }
    catch { close $guout }  
    # build the buttons and frame
    set w .tlChooser
    frame $w.fmLeft    
    pack $w.fmLeft -side left -pady 5 -padx 5
    if { $GraphicsLevels>8 } \
      { 
      frame $w.fmRight 
      pack $w.fmRight -side right -pady 5 -padx 5
      }
    for { set i 1 } { $i <= $GraphicsLevels } { incr i } \
      {
      if { $i <= 8 || ($i <= 16 && $GraphicsLevels>16 ) } \
        {
        # -- build the frame to hold the three buttons
        frame $w.fmLeft.fm$i
        set t $w.fmLeft.fm$i
        }\
      else \
        {
        # -- build the frame to hold the button and label
        frame $w.fmRight.fm$i
        set t $w.fmRight.fm$i
        }  
      #-- see if the button should be labeled or not
        if { $showlevel } { set tlabel $i } else { set tlabel "" } 
	label $t.lbColor -text $tlabel -width 2
        button $t.btColor -text [lindex $ColorList($i) 1] -foreground [InverseColor [lindex $ColorList($i) 0]] \
	                  -background [lindex $ColorList($i) 0] -width 12 \
			  -activebackground [lindex $ColorList($i) 0] \
			  -highlightthickness 2 -highlightcolor red \
			  -activeforeground [InverseColor [lindex $ColorList($i) 0]] \
	                  -command "set CCLevel $i; destroy $w" 

#        button $t.btColor -bitmap  @[bitmapLoc cursor_solid.xbm] -foreground $ColorList($i) 
#	                  -activeforeground $ColorList($i) 
#			  -highlightthickness 2 -highlightcolor red 
#	                  -command "set CCLevel $i; destroy $w" 
        pack $t.lbColor -side left		       
        pack $t.btColor -side left -padx 5
        pack $t -side top -padx 4 
	if { $i == $lev } { focus $t.btColor }
      }

  tkwait window .tlChooser

  return $CCLevel
  }






