# Copyright(c) 1999, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: PointSelect.tcl,v 1.18 2005/08/19 20:43:24 kevinb Rel $

#-----------------------------------------------------------------------------------
#
#
#===================================================================================#
#-----------------------------------------------------------------------------------#
# proc PointSelect { } -- Upper level code to organize the PTDISP command
#-----------------------------------------------------------------------------------#
proc PointSelect { CommandType } \
   {
   #---- Make sure that the arrays are defined
   global PSCommandType
   
   global PSActiveServer PSActiveEntry
   global PSEntries 
   global PSTime
   
   global PSLBox     ; # different that Grids/Images -- just a description field
   global PSMenues   ; # Holds frame/window path so can update the menues on the fly
   global PSProjection PSParameter PSLevel 
   global PSUnits
   global PSUnitColor PSMapColor 
   global PSLabelHeight PSLabelColor

   if { [winfo exists .tlPointSelect] } \
   { 
	wm deiconify .tlPointSelect
	focus .tlPointSelect
	raise .tlPointSelect
	return
   }

   #--- Initialize the list data correctly (use last active Server if applicable else first in list)
     if { ! [info exists PSActiveServer] } { set PSActiveServer 0 }
     if { ! [info exists PSActiveEntry] }  { set PSActiveEntry 0 }
     PSBuildList

   #====================== Initialize all of the data ===========================
     set PSSource ""			
     set PSProjection ""		
     set PSLevel ""
     set PSLabelHeight 8
     set PSLabelColor 3
     set PSMapColor 1
     set PSMapOptions "NAV=C"
     set PSCommandType $CommandType

   toplevel .tlPointSelect -borderwidth 3 -relief raised
   wm title .tlPointSelect "Point Data Select"
 
   frame .tlPointSelect.fmBody
     Notebook:create .tlPointSelect.nbPointDisp -pages { Data Options } -pad 10\
        -width 500 -height 500
     pack .tlPointSelect.nbPointDisp -fill both -expand true
     #---- DATA section ---------------------------------------
     set data [Notebook:frame .tlPointSelect.nbPointDisp Data]
       #---- Menues frame
       frame $data.frameMenu
         LMCreateLoopButton $data.frameMenu "Select Frame" YES
         #--- Dataset menu
         frame $data.frameMenu.frameDS
	 pack $data.frameMenu.frameDS -side top -anchor w -fill x 
         #-- Parameter frame
         frame $data.frameMenu.frameParameter
         pack $data.frameMenu.frameParameter -side top -anchor w -fill x
         #-- Level frame
         frame $data.frameMenu.frameLevel
         pack $data.frameMenu.frameLevel -side top -anchor w -fill x
         #-- Units frame
         frame $data.frameMenu.frameUnits
         pack $data.frameMenu.frameUnits -side top -anchor w -fill x
	 #--- Time 
         frame $data.frameMenu.frameTime
	 pack $data.frameMenu.frameTime -side top -anchor w -fill x 
         #--Listbox frame      	      
         frame $data.frameMenu.frameList
           #-- The List box -- need PSLBox to be defined for DataSet section
           set PSLBox [MakeListBox $data.frameMenu.frameList \
                      "    Pos  Description" ""]
           #$PSLBox config -selectmode extended 
	 pack $data.frameMenu.frameList -side top -fill both -expand true
       pack $data.frameMenu -side top -pady 10 -padx 10 -fill both -expand true
       set PSMenues $data.frameMenu
	 
     #---- OPTIONS section ---------------------------------------

     set options [Notebook:frame .tlPointSelect.nbPointDisp Options]
       #-- Main frame
       frame $options.frameMenu
       #-- Map Options
         frame $options.frameMenu.frameMapOptions -relief raised -borderwidth 3
           PSMapOptionsMenu $options.frameMenu.frameMapOptions 
	 pack $options.frameMenu.frameMapOptions  -side top -fill x
       #--Map Color
         frame $options.frameMenu.frameMapColor -relief raised -borderwidth 3
            button $options.frameMenu.frameMapColor.buttonSelectColor -text "Map Color" -borderwidth 2 -width 17 \
      		-command {set PSMapColor [ColorChooser $PSMapColor]
		          set options [Notebook:frame .tlPointSelect.nbPointDisp Options]
                          $options.frameMenu.frameMapColor.labelColor config -bg [ColorLevelToRGB $PSMapColor]}
            label $options.frameMenu.frameMapColor.labelColor \
		-highlightthickness 0 \
		-relief groove \
		-width 2 \
		-bg [ColorLevelToRGB $PSMapColor]
            pack $options.frameMenu.frameMapColor.buttonSelectColor -side left 
            pack $options.frameMenu.frameMapColor.labelColor -side right -fill x -expand true -padx 2
         pack $options.frameMenu.frameMapColor -side top -fill x 
       #--Label Height
         frame $options.frameMenu.frameLHeight -relief raised -borderwidth 3
           PSLabelMenu $options.frameMenu.frameLHeight
         pack $options.frameMenu.frameLHeight -side top -fill x
       #--Label  Color
         frame $options.frameMenu.frameContourColor -relief raised -borderwidth 3
            button $options.frameMenu.frameContourColor.buttonSelectColor \
		-text "Label Color" \
		-borderwidth 2 \
		-width 17 \
      		-command {set PSLabelColor [ColorChooser $PSLabelColor]
		          set options [Notebook:frame .tlPointSelect.nbPointDisp Options]
                          $options.frameMenu.frameContourColor.labelColor config \
					-bg [ColorLevelToRGB $PSLabelColor]}
            label $options.frameMenu.frameContourColor.labelColor \
		-highlightthickness 0 \
		-relief groove \
		-width 2 \
		-bg [ColorLevelToRGB $PSLabelColor]
            pack $options.frameMenu.frameContourColor.buttonSelectColor -side left 
            pack $options.frameMenu.frameContourColor.labelColor -side right -fill x -expand true -padx 2
         pack $options.frameMenu.frameContourColor -side top -fill x 
       #--Projection
         frame $options.frameMenu.frameProjection -relief raised -borderwidth 3
           PSProjectionMenu $options.frameMenu.frameProjection
	 pack  $options.frameMenu.frameProjection -side top -fill x
       pack $options.frameMenu -side top -pady 10 -padx 10 -fill both -expand true
   pack .tlPointSelect.fmBody  
   

   #---Controls frame (load/dismiss/load&dismiss)
   frame .tlPointSelect.fmBottom 
     button .tlPointSelect.fmBottom.btLoad -text "Load Point(s)" -command PSLoadPoints
     button .tlPointSelect.fmBottom.btLoadDismiss -text "Load and Dismiss" -command  { PSLoadPoints; destroy .tlPointSelect }
     button .tlPointSelect.fmBottom.btDismiss -text "Dismiss" -command {destroy .tlPointSelect }
     pack .tlPointSelect.fmBottom.btLoad -side left -padx 10 -pady 5
     pack .tlPointSelect.fmBottom.btLoadDismiss -side left -padx 10 -pady 5
     pack .tlPointSelect.fmBottom.btDismiss -side right -padx 10 -pady 5
   pack .tlPointSelect.fmBottom -padx 20 -pady 5


   #------------ Fill the menues here ----------------------------
     set f $data.frameMenu.frameDS
     set enPSDataSet ""
     
     menubutton $f.mb -text "Dataset" -menu $f.mb.menu \
                      -relief raised -borderwidth 2 -indicatoron 1 -width 12
     pack  $f.mb -side left
     label $f.entryDS -textvar enPSDataSet -width 26 -anchor w -relief sunken -borderwidth 2
     pack  $f.entryDS -side left -fill x -expand true
     set m [menu $f.mb.menu -tearoff 0]
     set ServerIndex 0
     while { [info exists PSEntries($ServerIndex)] } \
       {
       $m add cascade -label $PSEntries($ServerIndex) -menu $m.sub$ServerIndex 
       set EntryIndex 0
       set m2 [menu $m.sub$ServerIndex -tearoff 0]
       while { [info exists PSEntries($ServerIndex,$EntryIndex)] }\
         {
         $m2 add command -label $PSEntries($ServerIndex,$EntryIndex) -command "PSDoList $ServerIndex $EntryIndex" 
	 incr EntryIndex
         }
       incr ServerIndex   
       }   
   
    #--Parameter Menu
      set f $data.frameMenu    
      set PSParameter ""
      menubutton $f.frameParameter.mb -text "Parameter" -menu $f.frameParameter.mb.menu \
                                 -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameParameter.mb -side left
      label $f.frameParameter.entryParam \
		-textvar PSParameter -width 26 -anchor w -relief sunken -borderwidth 2
      pack  $f.frameParameter.entryParam -side left -fill x -expand true
      pack  $f.frameParameter -side top -fill x 
      set m [menu $f.frameParameter.mb.menu -tearoff 0]
    #--Selections Menu  -- was Level
      menubutton $f.frameLevel.mb -text "Level" -menu $f.frameLevel.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameLevel.mb -side left
      label $f.frameLevel.entryLevel \
		-textvar PSLevel -width 26 -anchor w -relief sunken -borderwidth 2
      pack  $f.frameLevel.entryLevel -side left -fill x -expand true
      pack  $f.frameLevel -side top -fill x 
      set m [menu $f.frameLevel.mb.menu -tearoff 0]

    #-- Units Menu
      menubutton $f.frameUnits.mbCU -text "Units" -relief raised -width 12 -borderwidth 2 -indicatoron 1 \
                       -menu $f.frameUnits.mbCU.menu
      #-- now the entry to echo the value
      label $f.frameUnits.entryUnits \
		-textvariable PSUnits -width 26 -anchor w -relief sunken -borderwidth 2
      #-- pack it
      pack $f.frameUnits.mbCU -side left
      pack $f.frameUnits.entryUnits -side left -fill x -expand true

    #--Time Menu
      menubutton $f.frameTime.mb -text "Time" -menu $f.frameTime.mb.menu \
                               -relief raised -borderwidth 2 -indicatoron 1 -width 12
      pack  $f.frameTime.mb -side left
      label $f.frameTime.entryTime \
		-textvar PSTime -width 26 -anchor w -relief sunken -borderwidth 2
      pack  $f.frameTime.entryTime -side left -fill x -expand true
      pack  $f.frameTime -side top -fill x 
      set m [menu $f.frameTime.mb.menu -tearoff 0]
    }
#-----------------------------------------------------------------------------------#   
# proc PSBuildList -- fills the PSEntries array
#-----------------------------------------------------------------------------------#
proc PSBuildList { } \
   {
   #--set up correct access to the arrays
   global PSEntries 
   global ServerList ServerListLength

   catch { unset PSEntries }
  
   #--set up correct access to the arrays
   set ServerIndex 0
   
   for { set i 0 } { $i < $ServerListLength } { incr i } \
     {
     if { [string match "P" [lindex $ServerList($i-H) 4] ] } \
       {
       # Have a valid POINT SERVER so add to the list
       set PSEntries($ServerIndex) [lindex $ServerList($i-H) 0]
       set descriptors [lindex $ServerList($i-H) 7] 
       set EntryIndex 0
       for { set j 0 } { $j<$descriptors } { incr j } \
         {
	 set line $ServerList($i-$j)
	 if { [string match "P" [lindex $line 0 ] ] } \
	   {
           set PSEntries($ServerIndex,$EntryIndex) [format "%-10s %-20s" [lindex $line 1] [lrange $line 2 end]]
	   incr EntryIndex
	   }
  	 }
       incr ServerIndex	 
       }
     }
   }
#---------------------------------------------------------------------------------#
# PSBuildMenues { } Fills the menus with valid ranges
#---------------------------------------------------------------------------------#
proc PSBuildMenues { } \
   {
   global PSCommandType

    #--- Later use first entry after "Number of ..." line as the LEVEL selector
    #--- and ignore the following variables as parameters (ptlist ... form=param)
    
    #--- add format= selection (PSfmt and selectable??)
    
    #--- for general case allow ALL Parameters to come through



   global PSEntries
   global PSActiveServer PSActiveEntry
   global PSLBox
   global PSMenues
   global PSParameter PSLevel PSTime
   global PSfmt 
   global PSLastType
  # -- default color settings   
   global back                ; # --- background color
   global activeback          ; # --- active background color
   global fore                ; # --- foreground color
   global activefore          ; # --- active foreground    
   
  # --- intiialize LastType 
    if { ![info exists PSLastType] } { set PSLastType "" }
   
  # --- time for the watch
     showBusyStatus
  #--- access to the menu frame
    set f $PSMenues
  #--- clear the list box
   $PSLBox delete 0 end

  #--- Set up the Parameter Menu first
    set Group $PSEntries($PSActiveServer)
    set DataSet [lindex [string trimleft $PSEntries($PSActiveServer,$PSActiveEntry)] 0]
    
    set FileID [open "|ptlist.k $Group/$DataSet.-1 FORM=PARAM" r]
    #--- Verify logged on
      gets $FileID line
      if { [string first "Accounting data was not valid" $line] != -1 }  \
         {
         tk_messageBox -title "Server Error" -message "You are not logged on. \nPlease log on and try again"\
         		-type ok -parent . -icon warning
         catch { close $FileID }			
         return		
         }
    # -- logged on but need to burn one more line
    gets $FileID line
    set count 0
      
    while { [gets $FileID line] != -1 } \
      {
      if { [string match "ptlist.k: Done" $line] } { break }
      #-- Parameter
	set Params($count,P) [string trim [string range $line 3 11]]
      #-- Format
	set Params($count,F) [string trim [string range $line 42 55]]
        incr count
      }

    catch { close $FileID }	

  #-- Build up the Parameter menu
    catch { destroy $f.frameParameter.mb.menu }
    set m [menu $f.frameParameter.mb.menu -tearoff 0]
    
    set ulist ""
#    set selecttypes ""
    set mcount 0
    for { set i 0 } { $i<$count } { incr i } \
      {
      #--- when we have reached the "Number of Repeat Groups=" line it
      #--- is time to generate that selection list as well 
      # --- going overboard -- comment out 
      #if { [string match $Params($i,P) "Number of"] } \
      #  { 
      #	# -- some data sets will NOT include this line -- RTPRSRC/SRFHOURLY for example
      #	incr i
      #	for { set j $i } { $j<$count } { incr j } { lappend selecttypes $Params($j,P) }
      #	break  ;# and break free of the loop
      #	}
      switch -- $Params($i,P)\
        {
        CIGH { set text "Ceiling Height" 
	       set UnitType "LINEAR" }
	CIGC { set text "Cloud Cover of Ceiling" 
	       set UnitType "LINEAR" }
	Z    { set text "Height???--Z" 
	       set UnitType "LINEAR" }
	TD   { set text "Dewpoint" 
	       set UnitType "TEMPERATURE" }
	PSL  { set text "Pressure at Sea Level" 
	       set UnitType "PRESSURE" }
	PRE  { set text "Pressure at Station" 
	       set UnitType "PRESSURE" }
	P    { set text "Pressure???--P" 
               set UnitType "PRESSURE" }
	PCP  { set text "Precipitation" 
	       set UnitType "LINEAR" }
	SNO  { set text "Snow" 
	       set UnitType "LINEAR" }
	ZS   { set text "Station Elevation" 
	       set UnitType "LINEAR" }
	T    { set text "Temperature" 
	       set UnitType "TEMPERATURE" }
	VIS  { set text "Visibility" 
	       set UnitType "LINEAR" }
	WXSYMB { set text "Weather Symbol" 
 	         set UnitType "WIND" }
	WXTEXT { set text "Weather Text" 
	         set UnitType "WIND" }
	DIR    { set text "Wind Direction" 
  	         set UnitType "WIND"}
 	WIND   { set text "Wind Flag" 
 	         set UnitType "WIND" }
	GUS    { set text "Wind Gusts" 
	         set UnitType "WIND" }
	SPD    { set text "Wind Speed" 
                 set UnitType "WIND" }
	default { if { [string match $PSCommandType "ALL"] } \
	            {
		    set text $Params($i,P)
		    set UnitType "ALL"
		    } \
	          else \
		    {
		    #puts stdout "Missed $Params($i,P)"
  	            set text ""
	            set UnitType "" 
		    }
		}   
        }
      if { [string length $text] } \
	{  
	if { $mcount==30 } \
	  {
          $m add cascade -label "MORE" -menu $m.sub 
          set m [menu $m.sub -tearoff 0]
	  set mcount 0
	  }
        $m add radio -label $text -variable PSParameter -value $Params($i,P) \
                     -command   "set PSfmt $Params($i,F)
			         PSUnitsMenu $UnitType 
      		                 set PSLastType $UnitType" 
	append ulist " $UnitType"
	incr mcount
        }		     
      }
    if { ![llength $ulist] } \
      {
      set PSParameter ""
      set PSUnits ""
      $f.frameParameter.mb configure -foreground grey -activeforeground grey -activebackground grey 
      } \
    else \
      {  
      $f.frameParameter.mb configure -foreground $fore -activeforeground $activefore -activebackground $activeback
      }
  #--- Units Menu
    if { [lsearch -glob $ulist $PSLastType]>=0 } { PSUnitsMenu $PSLastType } else { PSUnitsMenu "" }
      
  #--- Now build up the LEVEL/PRESSURE etc... menu
    catch { destroy $f.frameLevel.mb.menu }
    set m [menu $f.frameLevel.mb.menu -tearoff 0]
    set i 0
  
    # --- Alter this so that if Upper Air data is being used to build up a specific list
    # --- Otherwise deactivate the button completely (text to grey)
    
    if { [string match $PSCommandType "UPPERAIR"] } \
      {
      $f.frameLevel.mb configure -foreground $fore -activeforeground $activefore -activebackground $activeback
      foreach value { "SFC" "100" "150" "200" "250" "300" "400" "500" "700" "850" "925" "1000" "MAXW" "TRO1" } \
        {
        $m add radio -label $value -variable PSLevel -value "LEV $value"
        }
      } \
    else \
      {
      # --- set text to grey
      $f.frameLevel.mb configure -foreground grey -activeforeground grey -activebackground grey 
      }  

#  --- was going overboard -- just do the LEVEL parameter
#    puts stdout "Building LEVEL/PRESSURE menu"
#    puts stdout $selecttypes
#    foreach stype $selecttypes \
#      {
#      puts stdout  "Looking for selection for $stype"
#      $m add cascade -label $stype -menu $m.sub$i 
#      set m2 [menu $m.sub$i -tearoff 0]
#      incr i
#      
#      set FileID [open "|ptlist.k $Group/$DataSet.-1" r]
#
#      set levellist ""
#      while { [gets $FileID line] != -1 } \
#        {
#	if { [string match "ptlist.k: Done" $line] } { break }
#        #--- string has 2 possible LEV(etc..) locations check left first
#	if { [string match [string trim [string range $line 0 2]] $stype] } \
#	  {
#	  set value [string range $line 13 23]
#	  # if the value isn't "_missing_" then go ahead and append it
#	  if { [lsearch -glob $levellist $value ] <0 && 
#              ![string match [string trim $value] "_missing_"] } \
#	    {
#	    lappend levellist $value
#	    }
#	  }
#	#---- and right  
#	if { [string match [string range $line 38 40] $stype] } \
#	  {
#	  set value [string range $line 51 61]
#	  if { [lsearch -glob $levellist $value ]<0 && 
#              ![string match [string trim $value] "_missing_"] } \
#	    {
#	    lappend levellist $value
#	    }
#	  }
#	}  
#      catch { close $FileID }	
#      if { ![llength $levellist] } \
#	{
#	set PSLevel ""
#        $f.frameLevel.mb configure -foreground grey -activeforeground grey -activebackground grey 
#	} \
#      else \
#	{  
#        $f.frameLevel.mb configure -foreground $fore -activeforeground $activefore -activebackground $activeback
#	}
#	  
#    #-- Sort the list and insert all of the elements into the level list
#      set levellist [lsort $levellist]
#      foreach value $levellist \
#        {
#	set level [string trim $value]
#        $m2 add radio -label $level -variable PSLevel -value "$stype $level"
#        }
#      }	
#  
    #--- finally set up the TIME stuff   
      catch { destroy $f.frameTime.mb.menu }
      set m [menu $f.frameTime.mb.menu -tearoff 0]
  
      #-- Get the current time and use that as the default
        set tempPSTime [Now]
      # get first 2 characters out of it that contain the hour
        set PSTime [string range $tempPSTime 0 1]
        if { [string match $PSCommandType "UPPERAIR"] } \
          { 
          if {$PSTime > 0 && $PSTime < 12 } { set PSTime 0 } else { set PSTime 12 }
  	  set Iamount 1 ;# 12 if want just 0 and 12
	  } \
        else  { set Iamount 1 }
      
        for { set i 0} { $i < 24 } { incr i $Iamount } \
          {
          $m add radio -label $i -variable PSTime
	  }
    #-- now fill the list box with descriptions
      set FileID [open "|ptlist.k $Group/$DataSet FORM=FILE ALL" r]
      gets $FileID line    ;# Header
      gets $FileID line    ;# Dashed spacing
      set F 0
      while { [gets $FileID line] != -1 } \
        {
	if { [string match "ptlist.k: Done" $line] } { break }
        $PSLBox insert $F [string range $line 0 40]
	incr F
        }
      catch { close $FileID } 

  # --- restore cursor
    showIdleStatus      
  }

#-----------------------------------------------------------------------------------------#
# proc PSDoList { ServerIndex EntryIndex  } -- get a detailed Grid list at this location #
#	-- ServerIndex -- server to use							  #
#	-- EntryIndex  -- DataSet in the Server to use					  #
#-----------------------------------------------------------------------------------------#
proc PSDoList { ServerIndex EntryIndex } \
  {
  global PSEntries
  global enPSDataSet
  global PSActiveServer PSActiveEntry
    
  # -- set the Server and Entry correctly
    set PSActiveServer $ServerIndex
    set PSActiveEntry $EntryIndex
      
  # -- set the label header here
    set Group $PSEntries($PSActiveServer)
    set enPSDataSet "$Group -- [string trimleft $PSEntries($PSActiveServer,$PSActiveEntry)]" 
    
  # -- update the sub-selectors now
    PSBuildMenues 
  }
#------------------------------------------------------------------------------------#
# PSUnitsMenu { } -- builds the units selection
#------------------------------------------------------------------------------------#
proc PSUnitsMenu { UnitType } \
  {
  global PSUnits 
  global PSMenues
  global PSLastType    

  
  if { ![string match $UnitType $PSLastType] } { set PSUnits "" }

  set f $PSMenues.frameUnits

  catch {destroy $f.mbCU.menu }
  menu $f.mbCU.menu -tearoff 0
  
  #-- Linear Units here
  if { [string match $UnitType "LINEAR" ] || [string match $UnitType "ALL"] } \
    {
    $f.mbCU.menu add cascade -label "Linear Measure" -menu $f.mbCU.menu.lin
      set m [menu $f.mbCU.menu.lin -tearoff 0]
      $m add radio -label "Centimeters (CM)" -value "CM" -variable PSUnits
      $m add radio -label "Decameters (DM)"  -value "DM" -variable PSUnits
      $m add radio -label "Feet (FT)"        -value "FT" -variable PSUnits
      $m add radio -label "Geopotential Meters (GPM)"    -value "GPM" -variable PSUnits
      $m add radio -label "Inches (IN)"      -value "IN" -variable PSUnits
      $m add radio -label "Kilometers (KM)"      -value "KM" -variable PSUnits
      $m add radio -label "Meters (M)"       -value "M"  -variable PSUnits
      $m add radio -label "Miles (MI)"       -value "MI" -variable PSUnits
      $m add radio -label "Millimeters (MM)" -value "MM" -variable PSUnits
      $m add radio -label "Nautical Miles (NMI)"         -value "NMI" -variable PSUnits
      $m add radio -label "Yards (YD)"       -value "YD" -variable PSUnits
    }

  #-- Pressure Units here
  if { [string match $UnitType "PRESSURE" ] || [string match $UnitType "ALL"] } \
    {
    $f.mbCU.menu add cascade -label "Pressure"       -menu $f.mbCU.menu.pres
      set m [menu $f.mbCU.menu.pres -tearoff 0]
      $m add radio -label "Inches of Mercury (INHG)" -value "INHG" -variable PSUnits
      $m add radio -label "Hectopascals (HPA)"       -value "HPA"  -variable PSUnits
      $m add radio -label "Millibars (MB)"           -value "MB"   -variable PSUnits
    }  

  #-- Temperature Units here
  if { [string match $UnitType "TEMPERATURE" ] || [string match $UnitType "ALL"] } \
    {
    $f.mbCU.menu add cascade -label "Temperature"    -menu $f.mbCU.menu.temp
      set m [menu $f.mbCU.menu.temp -tearoff 0]
      $m add radio -label "Celsius (C)"     -value "C" -variable PSUnits
      $m add radio -label "Fahrenheit (F)"  -value "F" -variable PSUnits
      $m add radio -label "Kelvin (K)"      -value "K" -variable PSUnits
    }  

  #-- Wind Units here
  if { [string match $UnitType "WIND" ] || [string match $UnitType "ALL"] } \
    {
    $f.mbCU.menu add cascade -label "Wind"           -menu $f.mbCU.menu.wind
      set m [menu $f.mbCU.menu.wind -tearoff 0]
      $m add radio -label "Knots (KTS)"                -value "KTS" -variable PSUnits
      $m add radio -label "Meters Per Second (MPS)"    -value "MPS" -variable PSUnits
      $m add radio -label "Miles Per Hour (MPH)"       -value "MPH" -variable PSUnits
    }
    
  $f.mbCU.menu add command -label "DEFAULT" -command "set PSUnits DEFAULT"
  }

#---------------------------------------------------------------------------#
# proc PSLoadPoints -- loads the selected data set                          #
#---------------------------------------------------------------------------#
proc PSLoadPoints { } \
   {
   global PSEntries
   global PSActiveServer PSActiveEntry
   global PSLBox
   global PSMenues
   global PSParameter PSLevel PSTime PSUnits PSProjection
   global PSfmt 
   global PSMapOptions PSMapColor
   global PSLabelHeight PSLabelColor
   
  #--- Set up the Parameter Menu first
    set Group $PSEntries($PSActiveServer)
    set DataSet [lindex [string trimleft $PSEntries($PSActiveServer,$PSActiveEntry)] 0]

    if { [string length $PSLevel] } \
      {
      set select "SELECT='TIME $PSTime ;$PSLevel' "
      } \
    else \
      {
      set select "SELECT='TIME $PSTime' "
      }
    if { [string match $PSUnits "DEFAULT"] } \
      {
      set paramkey "PARAM=$PSParameter\[\]"
      }\
    else \
      {
      set paramkey "PARAM=$PSParameter\[$PSUnits\]"
      }

    set LoadOptions "$select $paramkey"
   #--build up the options here
    if { [string match "*NAV=C*" $PSMapOptions] } \
    {
	append LoadOptions " NAV=C"
    } \
    else \
    {
	append LoadOptions " MAP=$PSMapOptions"
    }
    append LoadOptions " MCOLOR=$PSMapColor"
    append LoadOptions " LSIZE=$PSLabelHeight"
    append LoadOptions " COLOR=$PSLabelColor"
   #-- Projection string  
     if { ![string match $PSProjection "DEFAULT"] } \
       {
       append LoadOptions " PRO=$PSProjection"
       }

   #--Add formatting option to restrict numberical output to integer values
     if {[string match $PSParameter "PCP"]} \
     {
	append LoadOptions " FORMAT=F5.2"
     } \
     else \
     {
	append LoadOptions " FORMAT=I10"
     }

#       {
#     if { [string match $MGSProjectionSet "MERC"] } { append LoadOptions " PRO=MERC" } \
#     elseif { [string match $MGSProjectionSet "LAMB"] }\
#       {
#       if { $MGSLat < 0 } { append LoadOptions " PRO=CONF SLON=$MGSLon SLAT=-30 -50"} \
#       else { append LoadOptions " PRO=CONF SLON=$MGSLon SLAT=30 50" }
#       }\
#     elseif { [string match $MGSProjectionSet "PS"] } \
#       {
#       if { $MGSLat < 0 } { append LoadOptions " PRO=CONF SLON=$MGSLon SLAT=-60"} \
#       else { append LoadOptions " PRO=CONF SLON=$MGSLon SLAT=60" }
#       }
#     }  
       
   
   if { [$PSLBox curselection]>=0 } \
     {
     # --- time for the watch
       showBusyStatus
    
     set Position [lindex [$PSLBox get [$PSLBox curselection]]  0]
     LMLoadPointsIntoFrame $Group $DataSet $Position $LoadOptions 1

     # --- restore cursor
       showIdleStatus      
     }
  }   

#-------------------------------------------------------------------------------------------#
# PSMapOptionsMenu {} -- builds the MapOptions menu here
#-------------------------------------------------------------------------------------------#
proc PSMapOptionsMenu { f } \
  {
  global PSMapOptions
    
  menubutton $f.mbMO \
	-text "Map Boundaries" \
	-relief raised \
	-width 16 \
	-borderwidth 2 \
	-indicatoron 1 \
	-menu $f.mbMO.menu

  set m [menu $f.mbMO.menu -tearoff 0]
  
  $m add radio \
	-label "Current Navigation (NAV=C)" \
	-variable PSMapOptions \
	-value "NAV=C"
  $m add radio \
	-label "Midwest USA (MID)" \
	-variable PSMapOptions \
	-value MID 
  $m add radio \
	-label "North America (NA)" \
	-variable PSMapOptions \
	-value NA 
  $m add radio \
	-label "USA (USA)" \
	-variable PSMapOptions \
	-value USA 
  $m add cascade \
	-label "States (AL-MT)" \
	-menu $m.states1 
  $m add cascade \
	-label "States (NE-WY)" \
	-menu $m.states2 
  
  set m2 [menu $m.states1 -tearoff 0] 
  foreach state { "Alabama (AL)" "Alaska (AK)" "Arizona (AZ)"\
		"Arkansas (AR)" "California (CA)" "Colorado (CO)" "Connecticut (CT)" \
		"Delaware (DE)" "Florida (FL)" "Georgia (GA)" "Hawaii (HI)" "Idaho (ID)"\
		"Illinois (IL)" "Indiana (IN)" "Iowa (IA)" "Kansas (KS)" "Kentucky (KY)"\
		"Louisiana (LA)" "Maine (ME)" "Maryland (MD)" "Massachusetts (MA)"\
		"Michigan (MI)" "Minnesota (MN)" "Mississippi (MS)" "Missouri (MO)"\
		"Montana (MT)" }\
    {
    set value [string range [lindex $state [expr [llength $state] -1]] 1 2]
    $m2 add radio -label $state -variable PSMapOptions -value "$value" 
    }		
    
  set m2 [menu $m.states2 -tearoff 0] 
  foreach state {"Nebraska (NE)" "Nevada (NV)" "New Hampshire (NH)"\
		 "New Jersey (NJ)" "New Mexico (NM)" "New York (NY)" "North Carolina (NC)"\
		 "North Dakota (ND)" "Ohio (OH)" "Oklahoma (OK)" "Oregon (OR)"\
		 "Pennsylvania (PA)" "Rhode Island (RI)" "South Carolina (SC)" \
		 "South Dakota (SD)" "Tennessee (TN)" "Texas (TX)" "Utah (UT)" "Vermont (VT)"\
		 "Virginia (VA)" "Washington (WA)" "West Virginia (WV)" "Wisconsin (WI)"\
		 "Wyoming (WY)" } \
    {
    set value [string range [lindex $state [expr [llength $state] -1]] 1 2]
    $m2 add radio -label $state -variable PSMapOptions -value "$value" 
    }		
		 
    
  label $f.entryMapOptions \
	-textvariable PSMapOptions -width 26 -anchor w -relief sunken -borderwidth 2
  pack $f.mbMO -side left
  pack $f.entryMapOptions -side left -fill x -expand true
  pack $f -side top -fill x 
  }  
#---------------------------------------------------------------------------------------------#
# PSLabelMenu -- builds the Label menu
#---------------------------------------------------------------------------------------------#
proc PSLabelMenu { f } \
  {
  global PSLabelHeight
  
  menubutton $f.mbLH -text "Label Height" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbLH.menu

  set m [menu $f.mbLH.menu -tearoff 0]
  
  foreach interval { 0 4 6 8 9 10 11 12 13 14 15 16 17 18 19 20} \
    {
    $m add radio -label $interval -variable PSLabelHeight -value "$interval"
    }

  label $f.entryLHeight \
	-textvariable PSLabelHeight -width 26 -anchor w -relief sunken -borderwidth 2
  pack $f.mbLH -side left
  pack $f.entryLHeight -side left -fill x -expand true
  }
#---------------------------------------------------------------------------------------------#
# PSProjectionMenu -- builds the Projection menu
#---------------------------------------------------------------------------------------------#
proc PSProjectionMenu { f } \
  {
  global PSProjection
  
  menubutton $f.mbPJ -text "Projection" -relief raised -width 16 -borderwidth 2 -indicatoron 1 \
                     -menu $f.mbPJ.menu

  set m [menu $f.mbPJ.menu -tearoff 0]
  
  $m add radio -label "Default (DEFAULT)" -variable PSProjection -value "DEFAULT"
  $m add radio -label "Lambert Conformal (CONF)" -variable PSProjection -value "CONF"
  $m add radio -label "Mercator (MERC)" -variable PSProjection -value "MERC"

  label $f.entryProjection \
	-textvariable PSProjection -anchor w -relief sunken -borderwidth 2
  pack $f.mbPJ -side left
  pack $f.entryProjection -side left -fill x -expand true
  }

